
package window;

import diagrams.DecimalLabelFormat;
import diagrams.ValueFormatter;
import java.awt.Color;
import java.util.Observable;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;



/**
 *
 * @author B. Gaertner
 */
public class ApplicationModel extends Observable implements ChangeListener {

    private DefaultBoundedRangeModel amplitudeModel;
    private DefaultBoundedRangeModel periodeModel;
    private DefaultBoundedRangeModel translationXModel;
    private DefaultBoundedRangeModel translationYModel;
    private NotificationChannel amplitudeField, periodeField,
                                translationXField, translationYField;
    private int selektion;
    private boolean showNormalFn;
    StyleContext sc;

    public ApplicationModel ()
    {
        amplitudeModel = new DefaultBoundedRangeModel(4, 0, -40, 40);
        periodeModel = new DefaultBoundedRangeModel(1, 0, -40, 40);
        translationXModel = new DefaultBoundedRangeModel(0, 0, -50, 50);
        translationYModel = new DefaultBoundedRangeModel(0, 0, -10, 10);
        amplitudeField = new NotificationChannel();
        periodeField = new NotificationChannel();
        translationXField = new NotificationChannel();
        translationYField = new NotificationChannel();
        selektion = 1;
        showNormalFn = true;


        sc = new StyleContext();
        Style st1 = sc.addStyle("BasicAttributes", null);  // no parent
        StyleConstants.setFontSize(st1, 24);
        StyleConstants.setFontFamily(st1, "serif");

        Style st2 = sc.addStyle("AmplitudeStyle", st1);  // derived from st1
        StyleConstants.setForeground(st2, Color.RED);

        Style st3 = sc.addStyle("PeriodeStyle", st1);
        StyleConstants.setForeground(st3, Color.BLUE);

        Style st4 = sc.addStyle("DeltaXStyle", st1);
        StyleConstants.setForeground(st4, new Color(0, 128, 0));

        Style st5 = sc.addStyle("DeltaYStyle", st1);
        StyleConstants.setForeground(st5, Color.MAGENTA);

        Style st6 = sc.addStyle("FnStyle", st1);
        st6.addAttribute(StyleConstants.Italic, true);
    }

    public void notifyButtonSelection (int id) {
        selektion = id;
        this.setChanged();
        this.notifyObservers();
    }

    public void showNormalFn (boolean b) {
        showNormalFn = b;
        this.setChanged();
        this.notifyObservers();
    }

    public DefaultBoundedRangeModel getAmplitudeModel() {
        return amplitudeModel;
    }

    public DefaultBoundedRangeModel getPeriodeModel() {
        return periodeModel;
    }

    public DefaultBoundedRangeModel getTranslationXModel() {
        return translationXModel;
    }

    public DefaultBoundedRangeModel getTranslationYModel() {
        return translationYModel;
    }

    public int getSelection() {
        return selektion;
    }

    public boolean getShowNormalFn() {
        return showNormalFn;
    }

    public double getAmplitude() {
        return amplitudeModel.getValue()/4.0;
    }

    public double getPeriode() {
        return periodeModel.getValue()/8.0;
    }

    public double getDeltaX() {
        return translationXModel.getValue() / 4.0;
    }

    public double getDeltaY() {
        return translationYModel.getValue() / 4.0;
    }

    public double getParameter(KindOfValue vt) {
        switch (vt) {
            case AMPLITUDE:
                return getAmplitude();
            case PERIODE:
                return getPeriode();
            case DELTA_X:
                return getDeltaX();
            case DELTA_Y:
                return getDeltaY();
            default:
                return 0.0;
        }
    }

    public Observable getMessageChannel(KindOfValue vt) {
        switch (vt) {
            case AMPLITUDE:
                return amplitudeField;
            case PERIODE:
                return periodeField;
            case DELTA_X:
                return translationXField;
            case DELTA_Y:
                return translationYField;
            default:
                return null;
        }
    }

    public StyledDocument getFormula() {
        boolean useParentheses;
        useParentheses = (this.getPeriode() != 1.0) & (this.getDeltaX() != 0.0);
        ValueFormatter fmt = new DecimalLabelFormat("#0.00");
        DefaultStyledDocument document;
        document = new DefaultStyledDocument(sc);
        StyledDocumentCreator writer = new StyledDocumentCreator(document);
        writer.setStyle("BasicAttributes");
        writer.append("f(x) = ");
        writer.setStyle("AmplitudeStyle");
        writer.append(fmt.format(this.getAmplitude()));
        writer.setStyle("BasicAttributes");
        writer.append(" * ");
        writer.setStyle("FnStyle");
        writer.append((selektion == 1 ? "sin" : "cos"));
        writer.setStyle("BasicAttributes");
        writer.append("(");
        writer.setStyle("PeriodeStyle");
        writer.append(fmt.format(this.getPeriode()));
        writer.setStyle("BasicAttributes");
        if (useParentheses) {
            writer.append("(");
        }
        writer.append("x");
        if (this.getDeltaX() !=0) {
            writer.setStyle("DeltaXStyle");
            writer.append((this.getDeltaX() < 0 ? " - " : " + "));
            writer.append(fmt.format(Math.abs(this.getDeltaX())));
            writer.setStyle("BasicAttributes");
        }

        if (useParentheses) {
            writer.append(")");
        }
        writer.append(")");
        writer.setStyle("DeltaYStyle");
        writer.append((this.getDeltaY() < 0 ? " - " : " + "));
        writer.append(fmt.format(Math.abs(this.getDeltaY())));
        writer.setStyle("BasicAttributes");
        SimpleAttributeSet center = new SimpleAttributeSet();
        StyleConstants.setAlignment(center, StyleConstants.ALIGN_CENTER);
        document.setParagraphAttributes(0, document.getLength(), center, false);
        return document;
    }

    /**
     * Interface ChangeListener:
     * Diese Methode nimmt fr alle vier Regler Meldungen zu Wertnderungen an.
     * @param e
     */
    @Override
    public void stateChanged(ChangeEvent e) {
        JSlider slider = (JSlider)e.getSource();
        DefaultBoundedRangeModel m = (DefaultBoundedRangeModel) slider.getModel();
        if (m == amplitudeModel) {
            amplitudeField.markAsChanged();
            amplitudeField.notifyObservers();
        }
        if (m == periodeModel) {
            periodeField.markAsChanged();
            periodeField.notifyObservers();
        }
        if (m == translationXModel) {
            translationXField.markAsChanged();
            translationXField.notifyObservers();
        }
        if (m == translationYModel) {
            translationYField.markAsChanged();
            translationYField.notifyObservers();
        }
        this.setChanged();
        this.notifyObservers();
    }

}
